#!/usr/bin/python3
"""
See http://blog.tfiu.de/globaldokumente-in-libreoffice-zusammenfuhren.html
"""

import os
import re
import sys

from xml import sax
from xml.sax.handler import ContentHandler


STYLE_PAT = re.compile('text:style-name="([^"]*)"')


def escape_PCDATA(val):
	return (val
		).replace("&", "&amp;"
		).replace('<', '&lt;'
		).replace('>', '&gt;'
		).replace("\0", "&x00;")


def escape_attr_val(val):
	return '"%s"'%(escape_PCDATA(val).replace('"', '&quot;'))


def make_att_string(atts, ignoredAttrs=set()):
	att_string = " ".join(["%s=%s"%(key, escape_attr_val(value))
		for key, value in dict(atts).items() if not key in ignoredAttrs])
	if att_string:
		att_string = " "+att_string
	return att_string


class StyleSanitiser(ContentHandler):
	def __init__(self, style_map, dest_f):
		self.style_map = style_map
		self.already_written = set()
		self.skipping = []
		self.dest_f = dest_f
	
	def startElement(self, name, attrs):
		attrs = dict(attrs)
		if self.skipping:
			self.skipping.append(name)
			return

		if name=="style:style":
			style_name = attrs["style:name"]
			if style_name in self.style_map:
				dest_name = self.style_map[attrs["style:name"]]
				if dest_name in self.already_written:
					self.skipping = [name]
					return
				else:
					attrs["style:name"] = attrs["style:display-name"] = dest_name
					if "style:parent-style-name" in attrs:
						attrs["style:parent-style-name"] = self.style_map.get(
							attrs["style:parent-style-name"], "Textkörper")
					if "style:next-style-name" in attrs:
						attrs["style:next-style-name"] = self.style_map.get(
							attrs["style:next-style-name"], "Textkörper")

				self.already_written.add(dest_name)
				self.already_written.add(style_name)
			else:
				self.skipping = [name]
				return

		self.dest_f.write("<{}{}>".format(name, make_att_string(attrs)))
	
	def endElement(self, name):
		if self.skipping:
			self.skipping.pop()
		else:
			self.dest_f.write(f"</{name}>")

	def characters(self, chars):
		if not self.skipping:
			self.dest_f.write(escapePCDATA(chars))


def read_style_map(src_name):
	style_map = {}
	with open(src_name) as f:
		for ln_no, ln in enumerate(f):
			try:
				src_style, dest_style = ln.split("->")
				style_map[src_style.strip()] = dest_style.strip()
			except Exception as msg:
				sys.exit(f"In line {ln_no+1}: {msg}")
	return style_map


def sanitize_styles(style_map):
	with open("new-styles.xml", "w") as out_f:
		sani = StyleSanitiser(style_map, out_f)
		sax.parse("styles.xml", sani)
	os.rename("new-styles.xml", "styles.xml")


def sub_styles(tx, src_name, style_map):
	modified = STYLE_PAT.sub(
		lambda mat: 'text:style-name="{}"'.format(
			style_map.get(mat.group(1), mat.group(1))), text)
	os.rename(src_name, src_name+".bak")
	with open(src_name, "w", encoding="utf-8") as f:
		f.write(modified)


def list_styles(tx):
	print("\n".join(f"{s} -> {s}"
		for s in sorted(
			set(mat.group(1) 
				for mat in
					STYLE_PAT.finditer(tx)))))


if __name__=="__main__":
	to_process = "content.xml"
	with open(to_process, "r", encoding="utf-8") as f:
		text = f.read()

	if len(sys.argv)==2:
		sanitize_styles(read_style_map(sys.argv[1]))
		sub_styles(text, to_process, read_style_map(sys.argv[1]))
	elif len(sys.argv)==1:
		list_styles(text)
	else:
		sys.exit("usage: {sys.argv[0]} <style-map>")
